@@ -1,17 +1,13 @@
 """
 Interpolation inside triangular grids.
 """
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
-from matplotlib.externals import six
-from matplotlib.externals.six.moves import xrange
+import numpy as np
 
+from matplotlib import cbook
 from matplotlib.tri import Triangulation
 from matplotlib.tri.trifinder import TriFinder
 from matplotlib.tri.tritools import TriAnalyzer
-import numpy as np
-import warnings
 
 __all__ = ('TriInterpolator', 'LinearTriInterpolator', 'CubicTriInterpolator')
 
@@ -57,7 +53,7 @@ class TriInterpolator(object):
         self._unit_y = 1.0
 
         # Default triangle renumbering: None (= no renumbering)
-        # Renumbering may be used to avoid unecessary computations
+        # Renumbering may be used to avoid unnecessary computations
         # if complex calculations are done inside the Interpolator.
         # Please refer to :meth:`_interpolate_multikeys` for details.
         self._tri_renum = None
@@ -66,7 +62,7 @@ class TriInterpolator(object):
     # (except, if needed, relevant additions).
     # However these methods are only implemented in subclasses to avoid
     # confusion in the documentation.
-    docstring__call__ = """
+    _docstring__call__ = """
         Returns a masked array containing interpolated values at the specified
         x,y points.
 
@@ -79,13 +75,12 @@ class TriInterpolator(object):
         Returns
         -------
         z : np.ma.array
-            Masked array of the same shape as *x* and *y* ; values
-            corresponding to (*x*, *y*) points outside of the triangulation
-            are masked out.
+            Masked array of the same shape as *x* and *y*; values corresponding
+            to (*x*, *y*) points outside of the triangulation are masked out.
 
         """
 
-    docstringgradient = """
+    _docstringgradient = r"""
         Returns a list of 2 masked arrays containing interpolated derivatives
         at the specified x,y points.
 
@@ -98,12 +93,12 @@ class TriInterpolator(object):
         Returns
         -------
         dzdx, dzdy : np.ma.array
-            2  masked arrays of the same shape as *x* and *y* ; values
+            2 masked arrays of the same shape as *x* and *y*; values
             corresponding to (x,y) points outside of the triangulation
             are masked out.
             The first returned array contains the values of
-            :math:`\\frac{\\partial z}{\\partial x}` and the second those of
-            :math:`\\frac{\\partial z}{\\partial y}`.
+            :math:`\frac{\partial z}{\partial x}` and the second those of
+            :math:`\frac{\partial z}{\partial y}`.
 
         """
 
@@ -128,7 +123,7 @@ class TriInterpolator(object):
               unnecessary to compute the containing triangles twice)
             - scaling according to self._unit_x, self._unit_y
             - dealing with points outside of the grid (with fill value np.nan)
-            - dealing with multi-dimensionnal *x*, *y* arrays: flattening for
+            - dealing with multi-dimensional *x*, *y* arrays: flattening for
               :meth:`_interpolate_params` call and final reshaping.
 
         (Note that np.vectorize could do most of those things very well for
@@ -166,7 +161,7 @@ class TriInterpolator(object):
         x = np.asarray(x, dtype=np.float64)
         y = np.asarray(y, dtype=np.float64)
         sh_ret = x.shape
-        if (x.shape != y.shape):
+        if x.shape != y.shape:
             raise ValueError("x and y shall have same shapes."
                              " Given: {0} and {1}".format(x.shape, y.shape))
         x = np.ravel(x)
@@ -179,7 +174,7 @@ class TriInterpolator(object):
         if tri_index is None:
             tri_index = self._trifinder(x, y)
         else:
-            if (tri_index.shape != sh_ret):
+            if tri_index.shape != sh_ret:
                 raise ValueError(
                     "tri_index array is provided and shall"
                     " have same shape as x and y. Given: "
@@ -274,12 +269,12 @@ class LinearTriInterpolator(TriInterpolator):
     def __call__(self, x, y):
         return self._interpolate_multikeys(x, y, tri_index=None,
                                            return_keys=('z',))[0]
-    __call__.__doc__ = TriInterpolator.docstring__call__
+    __call__.__doc__ = TriInterpolator._docstring__call__
 
     def gradient(self, x, y):
         return self._interpolate_multikeys(x, y, tri_index=None,
                                            return_keys=('dzdx', 'dzdy'))
-    gradient.__doc__ = TriInterpolator.docstringgradient
+    gradient.__doc__ = TriInterpolator._docstringgradient
 
     def _interpolate_single_key(self, return_key, tri_index, x, y):
         if return_key == 'z':
@@ -295,7 +290,7 @@ class LinearTriInterpolator(TriInterpolator):
 
 
 class CubicTriInterpolator(TriInterpolator):
-    """
+    r"""
     A CubicTriInterpolator performs cubic interpolation on triangular grids.
 
     In one-dimension - on a segment - a cubic interpolating function is
@@ -370,11 +365,11 @@ class CubicTriInterpolator(TriInterpolator):
 
         .. math::
 
-            E(z) = \\ \\frac{1}{2} \\int_{\\Omega}   \\left(
-            \\left( \\frac{\\partial^2{z}}{\\partial{x}^2} \\right)^2 +
-            \\left( \\frac{\\partial^2{z}}{\\partial{y}^2} \\right)^2 +
-            2\\left( \\frac{\\partial^2{z}}{\\partial{y}\\partial{x}}
-            \\right)^2 \\right)  dx\\,dy
+            E(z) = \frac{1}{2} \int_{\Omega} \left(
+                \left( \frac{\partial^2{z}}{\partial{x}^2} \right)^2 +
+                \left( \frac{\partial^2{z}}{\partial{y}^2} \right)^2 +
+                2\left( \frac{\partial^2{z}}{\partial{y}\partial{x}} \right)^2
+            \right) dx\,dy
 
     If the case *kind* ='geom' is chosen by the user, a simple geometric
     approximation is used (weighted average of the triangle normal
@@ -417,10 +412,10 @@ class CubicTriInterpolator(TriInterpolator):
         self._z = self._z[~node_mask]
 
         # Computing scale factors
-        self._unit_x = np.max(compressed_x) - np.min(compressed_x)
-        self._unit_y = np.max(compressed_y) - np.min(compressed_y)
-        self._pts = np.vstack((compressed_x/float(self._unit_x),
-                               compressed_y/float(self._unit_y))).T
+        self._unit_x = np.ptp(compressed_x)
+        self._unit_y = np.ptp(compressed_y)
+        self._pts = np.column_stack([compressed_x / self._unit_x,
+                                     compressed_y / self._unit_y])
         # Computing triangle points
         self._tris_pts = self._pts[self._triangles]
         # Computing eccentricities
@@ -433,21 +428,12 @@ class CubicTriInterpolator(TriInterpolator):
     def __call__(self, x, y):
         return self._interpolate_multikeys(x, y, tri_index=None,
                                            return_keys=('z',))[0]
-    __call__.__doc__ = TriInterpolator.docstring__call__
+    __call__.__doc__ = TriInterpolator._docstring__call__
 
     def gradient(self, x, y):
         return self._interpolate_multikeys(x, y, tri_index=None,
                                            return_keys=('dzdx', 'dzdy'))
-    gradient.__doc__ = TriInterpolator.docstringgradient + """
-
-        Examples
-        --------
-        An example of effective application is shown below (plot of the
-        direction of the vector field derivated from a known potential field):
-
-        .. plot:: mpl_examples/pylab_examples/trigradient_demo.py
-
-        """
+    gradient.__doc__ = TriInterpolator._docstringgradient
 
     def _interpolate_single_key(self, return_key, tri_index, x, y):
         tris_pts = self._tris_pts[tri_index]
@@ -474,11 +460,11 @@ class CubicTriInterpolator(TriInterpolator):
 
         Parameters
         ----------
-        kind: {'min_E', 'geom', 'user'}
+        kind : {'min_E', 'geom', 'user'}
             Choice of the _DOF_estimator subclass to perform the gradient
             estimation.
-        dz: tuple of array_likes (dzdx, dzdy), optional
-            Used only if *kind=user ; in this case passed to the
+        dz : tuple of array_likes (dzdx, dzdy), optional
+            Used only if *kind*=user; in this case passed to the
             :class:`_DOF_estimator_user`.
 
         Returns
@@ -498,7 +484,7 @@ class CubicTriInterpolator(TriInterpolator):
         elif kind == 'min_E':
             TE = _DOF_estimator_min_E(self)
         else:
-            raise ValueError("CubicTriInterpolator *kind* proposed: {0} ; "
+            raise ValueError("CubicTriInterpolator *kind* proposed: {0}; "
                              "should be one of: "
                              "'user', 'geom', 'min_E'".format(kind))
         return TE.compute_dof_from_df()
@@ -526,12 +512,9 @@ class CubicTriInterpolator(TriInterpolator):
 
         a = tris_pts[:, 1, :] - tris_pts[:, 0, :]
         b = tris_pts[:, 2, :] - tris_pts[:, 0, :]
-        abT = np.concatenate([np.expand_dims(a, ndim+1),
-                              np.expand_dims(b, ndim+1)], ndim+1)
+        abT = np.stack([a, b], axis=-1)
         ab = _transpose_vectorized(abT)
-        x = np.expand_dims(x, ndim)
-        y = np.expand_dims(y, ndim)
-        OM = np.concatenate([x, y], ndim) - tris_pts[:, 0, :]
+        OM = np.stack([x, y], axis=1) - tris_pts[:, 0, :]
 
         metric = _prod_vectorized(ab, abT)
         # Here we try to deal with the colinear cases.
@@ -590,9 +573,9 @@ class CubicTriInterpolator(TriInterpolator):
               The so-called eccentricity parameters [1] needed for
               HCT triangular element.
         """
-        a = np.expand_dims(tris_pts[:, 2, :]-tris_pts[:, 1, :], axis=2)
-        b = np.expand_dims(tris_pts[:, 0, :]-tris_pts[:, 2, :], axis=2)
-        c = np.expand_dims(tris_pts[:, 1, :]-tris_pts[:, 0, :], axis=2)
+        a = np.expand_dims(tris_pts[:, 2, :] - tris_pts[:, 1, :], axis=2)
+        b = np.expand_dims(tris_pts[:, 0, :] - tris_pts[:, 2, :], axis=2)
+        c = np.expand_dims(tris_pts[:, 1, :] - tris_pts[:, 0, :], axis=2)
         # Do not use np.squeeze, this is dangerous if only one triangle
         # in the triangulation...
         dot_a = _prod_vectorized(_transpose_vectorized(a), a)[:, 0, 0]
@@ -793,7 +776,7 @@ class _ReducedHCT_Element():
         dsdksi = _roll_vectorized(prod, 3*subtri, axis=0)
         dfdksi = _prod_vectorized(dofs, dsdksi)
         # In global coordinates:
-        # Here we try to deal with the simpliest colinear cases, returning a
+        # Here we try to deal with the simplest colinear cases, returning a
         # null matrix.
         J_inv = _safe_inv22_vectorized(J)
         dfdx = _prod_vectorized(J_inv, _transpose_vectorized(dfdksi))
@@ -836,7 +819,7 @@ class _ReducedHCT_Element():
         Returns
         -------
         Returns the arrays d2sdksi2 (N x 3 x 1) Hessian of shape functions
-        expressed in covariante coordinates in first apex basis.
+        expressed in covariant coordinates in first apex basis.
         """
         subtri = np.argmin(alpha, axis=1)[:, 0]
         ksi = _roll_vectorized(alpha, -subtri, axis=0)
@@ -907,7 +890,7 @@ class _ReducedHCT_Element():
         pts = self.gauss_pts
         for igauss in range(self.n_gauss):
             alpha = np.tile(pts[igauss, :], n).reshape(n, 3)
-            alpha = np.expand_dims(alpha, 3)
+            alpha = np.expand_dims(alpha, 2)
             weight = weights[igauss]
             d2Skdksi2 = self.get_d2Sidksij2(alpha, ecc)
             d2Skdx2 = _prod_vectorized(d2Skdksi2, H_rot)
@@ -934,7 +917,7 @@ class _ReducedHCT_Element():
         to global coordinates.
         if *return_area* is True, returns also the triangle area (0.5*det(J))
         """
-        # Here we try to deal with the simpliest colinear cases ; a null
+        # Here we try to deal with the simplest colinear cases; a null
         # energy and area is imposed.
         J_inv = _safe_inv22_vectorized(J)
         Ji00 = J_inv[:, 0, 0]
@@ -976,7 +959,7 @@ class _ReducedHCT_Element():
         """
         ntri = np.size(ecc, 0)
         vec_range = np.arange(ntri, dtype=np.int32)
-        c_indices = -np.ones(ntri, dtype=np.int32)  # for unused dofs, -1
+        c_indices = np.full(ntri, -1, dtype=np.int32)  # for unused dofs, -1
         f_dof = [1, 2, 4, 5, 7, 8]
         c_dof = [0, 3, 6]
 
@@ -1080,9 +1063,9 @@ class _DOF_estimator():
         J1 = _prod_vectorized(_ReducedHCT_Element.J0_to_J1, J)
         J2 = _prod_vectorized(_ReducedHCT_Element.J0_to_J2, J)
 
-        col0 = _prod_vectorized(J, np.expand_dims(tri_dz[:, 0, :], axis=3))
-        col1 = _prod_vectorized(J1, np.expand_dims(tri_dz[:, 1, :], axis=3))
-        col2 = _prod_vectorized(J2, np.expand_dims(tri_dz[:, 2, :], axis=3))
+        col0 = _prod_vectorized(J, np.expand_dims(tri_dz[:, 0, :], axis=2))
+        col1 = _prod_vectorized(J1, np.expand_dims(tri_dz[:, 1, :], axis=2))
+        col2 = _prod_vectorized(J2, np.expand_dims(tri_dz[:, 2, :], axis=2))
 
         dfdksi = _to_matrix_vectorized([
             [col0[:, 0, 0], col1[:, 0, 0], col2[:, 0, 0]],
@@ -1094,7 +1077,8 @@ class _DOF_estimator():
 
 
 class _DOF_estimator_user(_DOF_estimator):
-    """ dz is imposed by user / Accounts for scaling if any """
+    """dz is imposed by user; accounts for scaling if any."""
+
     def compute_dz(self, dz):
         (dzdx, dzdy) = dz
         dzdx = dzdx * self._unit_x
@@ -1103,7 +1087,8 @@ class _DOF_estimator_user(_DOF_estimator):
 
 
 class _DOF_estimator_geom(_DOF_estimator):
-    """ Fast 'geometric' approximation, recommended for large arrays. """
+    """Fast 'geometric' approximation, recommended for large arrays."""
+
     def compute_dz(self):
         """
         self.df is computed as weighted average of _triangles sharing a common
@@ -1152,10 +1137,10 @@ class _DOF_estimator_geom(_DOF_estimator):
             alpha2 = np.arctan2(p2[:, 1]-p0[:, 1], p2[:, 0]-p0[:, 0])
             # In the below formula we could take modulo 2. but
             # modulo 1. is safer regarding round-off errors (flat triangles).
-            angle = np.abs(np.mod((alpha2-alpha1) / np.pi, 1.))
-            # Weight proportional to angle up np.pi/2 ; null weight for
-            # degenerated cases 0. and np.pi (Note that `angle` is normalized
-            # by np.pi)
+            angle = np.abs(((alpha2-alpha1) / np.pi) % 1)
+            # Weight proportional to angle up np.pi/2; null weight for
+            # degenerated cases 0 and np.pi (note that `angle` is normalized
+            # by np.pi).
             weights[:, ipt] = 0.5 - np.abs(angle-0.5)
         return weights
 
@@ -1171,7 +1156,7 @@ class _DOF_estimator_geom(_DOF_estimator):
         dM1 = tris_pts[:, 1, :] - tris_pts[:, 0, :]
         dM2 = tris_pts[:, 2, :] - tris_pts[:, 0, :]
         dM = np.dstack([dM1, dM2])
-        # Here we try to deal with the simpliest colinear cases: a null
+        # Here we try to deal with the simplest colinear cases: a null
         # gradient is assumed in this case.
         dM_inv = _safe_inv22_vectorized(dM)
 
@@ -1216,9 +1201,9 @@ class _DOF_estimator_min_E(_DOF_estimator_geom):
             J, eccs, triangles, Uc)
 
         # Building sparse matrix and solving minimization problem
-        # We could use scipy.sparse direct solver ; however to avoid this
+        # We could use scipy.sparse direct solver; however to avoid this
         # external dependency an implementation of a simple PCG solver with
-        # a simplendiagonal Jocabi preconditioner is implemented.
+        # a simple diagonal Jacobi preconditioner is implemented.
         tol = 1.e-10
         n_dof = Ff.shape[0]
         Kff_coo = _Sparse_Matrix_coo(Kff_vals, Kff_rows, Kff_cols,
@@ -1230,9 +1215,10 @@ class _DOF_estimator_min_E(_DOF_estimator_geom):
         err0 = np.linalg.norm(Kff_coo.dot(Uf0) - Ff)
         if err0 < err:
             # Maybe a good occasion to raise a warning here ?
-            warnings.warn("In TriCubicInterpolator initialization, PCG sparse"
-                          " solver did not converge after 1000 iterations. "
-                          "`geom` approximation is used instead of `min_E`")
+            cbook._warn_external("In TriCubicInterpolator initialization, "
+                                 "PCG sparse solver did not converge after "
+                                 "1000 iterations. `geom` approximation is "
+                                 "used instead of `min_E`")
             Uf = Uf0
 
         # Building dz from Uf
@@ -1265,12 +1251,9 @@ class _Sparse_Matrix_coo(object):
         *V* dense vector of shape (self.m,)
         """
         assert V.shape == (self.m,)
-        # For a more generic implementation we could use below kw argument
-        # minlength=self.m of bincount ; however:
-        # - it is new in numpy 1.6
-        # - it is unecessary when each row have at least 1 entry in global
-        #   matrix, which is the case here.
-        return np.bincount(self.rows, weights=self.vals*V[self.cols])
+        return np.bincount(self.rows,
+                           weights=self.vals*V[self.cols],
+                           minlength=self.m)
 
     def compress_csc(self):
         """
@@ -1334,14 +1317,14 @@ def _cg(A, b, x0=None, tol=1.e-10, maxiter=1000):
         Right hand side of the linear system.
 
     Returns
-    ----------
+    -------
     x: array.
         The converged solution.
     err: float
         The absolute error np.linalg.norm(A.dot(x) - b)
 
     Other parameters
-    ----------
+    ----------------
     x0: array.
         Starting guess for the solution.
     tol: float.
@@ -1360,7 +1343,7 @@ def _cg(A, b, x0=None, tol=1.e-10, maxiter=1000):
     # Jacobi pre-conditioner
     kvec = A.diag
     # For diag elem < 1e-6 we keep 1e-6.
-    kvec = np.where(kvec > 1.e-6, kvec, 1.e-6)
+    kvec = np.maximum(kvec, 1e-6)
 
     # Initial guess
     if x0 is None:
@@ -1394,7 +1377,6 @@ def _cg(A, b, x0=None, tol=1.e-10, maxiter=1000):
 
 
 # The following private functions:
-#     :func:`_inv22_vectorized`
 #     :func:`_safe_inv22_vectorized`
 #     :func:`_pseudo_inv22sym_vectorized`
 #     :func:`_prod_vectorized`
@@ -1405,20 +1387,6 @@ def _cg(A, b, x0=None, tol=1.e-10, maxiter=1000):
 #     :func:`_extract_submatrices`
 # provide fast numpy implementation of some standard operations on arrays of
 # matrices - stored as (:, n_rows, n_cols)-shaped np.arrays.
-def _inv22_vectorized(M):
-    """
-    Inversion of arrays of (2,2) matrices.
-    """
-    assert (M.ndim == 3)
-    assert (M.shape[-2:] == (2, 2))
-    M_inv = np.empty_like(M)
-    delta_inv = np.reciprocal(M[:, 0, 0]*M[:, 1, 1] - M[:, 0, 1]*M[:, 1, 0])
-    M_inv[:, 0, 0] = M[:, 1, 1]*delta_inv
-    M_inv[:, 0, 1] = -M[:, 0, 1]*delta_inv
-    M_inv[:, 1, 0] = -M[:, 1, 0]*delta_inv
-    M_inv[:, 1, 1] = M[:, 0, 0]*delta_inv
-    return M_inv
-
 
 # Development note: Dealing with pathologic 'flat' triangles in the
 # CubicTriInterpolator code and impact on (2,2)-matrix inversion functions
@@ -1468,7 +1436,7 @@ def _safe_inv22_vectorized(M):
     prod1 = M[:, 0, 0]*M[:, 1, 1]
     delta = prod1 - M[:, 0, 1]*M[:, 1, 0]
 
-    # We set delta_inv to 0. in case of a rank deficient matrix ; a
+    # We set delta_inv to 0. in case of a rank deficient matrix; a
     # rank-deficient input matrix *M* will lead to a null matrix in output
     rank2 = (np.abs(delta) > 1e-8*np.abs(prod1))
     if np.all(rank2):
@@ -1488,7 +1456,7 @@ def _safe_inv22_vectorized(M):
 
 def _pseudo_inv22sym_vectorized(M):
     """
-    Inversion of arrays of (2,2) SYMMETRIC matrices ; returns the
+    Inversion of arrays of (2,2) SYMMETRIC matrices; returns the
     (Moore-Penrose) pseudo-inverse for rank-deficient matrices.
 
     In case M is of rank 1, we have M = trace(M) x P where P is the orthogonal
@@ -1545,7 +1513,7 @@ def _prod_vectorized(M1, M2):
     assert sh1[-1] == sh2[-2]
 
     ndim1 = len(sh1)
-    t1_index = list(xrange(ndim1-2)) + [ndim1-1, ndim1-2]
+    t1_index = [*range(ndim1-2), ndim1-1, ndim1-2]
     return np.sum(np.transpose(M1, t1_index)[..., np.newaxis] *
                   M2[..., np.newaxis, :], -3)
 
@@ -1561,9 +1529,7 @@ def _transpose_vectorized(M):
     """
     Transposition of an array of matrices *M*.
     """
-    ndim = M.ndim
-    assert ndim == 3
-    return np.transpose(M, [0, ndim-1, ndim-2])
+    return np.transpose(M, [0, 2, 1])
 
 
 def _roll_vectorized(M, roll_indices, axis):
@@ -1605,7 +1571,7 @@ def _to_matrix_vectorized(M):
         M_res[...,i,j] = M[i][j]
     """
     assert isinstance(M, (tuple, list))
-    assert all([isinstance(item, (tuple, list)) for item in M])
+    assert all(isinstance(item, (tuple, list)) for item in M)
     c_vec = np.asarray([len(item) for item in M])
     assert np.all(c_vec-c_vec[0] == 0)
     r = len(M)
